#! python3  # noqa: E265


"""
    Check and set up the QGIS environment.
"""

# #############################################################################
# ########## Libraries #############
# ##################################

# Standard library
import json
import logging
from pathlib import Path
from timeit import default_timer

# 3rd party library
import click

from qgis_deployment_toolbelt.jobs.job_environment_variables import (
    JobEnvironmentVariables,
)

# submodules
from qgis_deployment_toolbelt.utils.bouncer import exit_cli_error, exit_cli_success

# #############################################################################
# ########## Globals ###############
# ##################################

# logs
logger = logging.getLogger(__name__)

# default CLI context.
# See: https://click.palletsprojects.com/en/7.x/commands/#context-defaults
CONTEXT_SETTINGS = dict(obj={})


# #############################################################################
# ####### Command-line ############
# #################################
@click.command(name="env-setup")
@click.option(
    "-d",
    "--deployment-configuration",
    help="Deployment scenario configuration (.json).",
    default="deployment-config.json",
    show_default=True,
    type=click.Path(
        exists=True, readable=True, file_okay=True, dir_okay=False, resolve_path=True
    ),
)
@click.pass_context
def environment_setup(cli_context: click.Context, deployment_configuration: Path):
    """Set up the system environment for QGIS.

    Args:
        cli_context (click.Context): Click context
    """
    # chronometer
    START_TIME = default_timer()
    logger.info("CHECK started after {:5.2f}s.".format(default_timer() - START_TIME))

    with Path(deployment_configuration).open("r") as f:
        scenario_data = json.load(f)

    env_manager = JobEnvironmentVariables()
    env_manager.run(scenario_data.get("environment_variables"))

    # ending
    exit_cli_success(
        message="CHECK completed after {:5.2f}s.".format(default_timer() - START_TIME),
        abort=False,
    )


# #############################################################################
# ##### Stand alone program ########
# ##################################

if __name__ == "__main__":
    """Standalone execution."""
    # launch cli
    environment_setup(obj={})



#! python3  # noqa: E265

"""
    Define toolbelt constant types and values.

    Author: Julien Moura (https://github.com/guts)
"""


# #############################################################################
# ########## Libraries #############
# ##################################

# Standard library
from os.path import expandvars
from pathlib import Path
from posixpath import expanduser
from typing import NamedTuple

# #############################################################################
# ########## Classes ###############
# ##################################


class OSConfiguration(NamedTuple):
    """Settings related to QGIS and depending on operating system"""

    profiles_path: Path


# #############################################################################
# ########## Classes ###############
# ##################################

OS_CONFIG: dict = {
    "darwin": OSConfiguration(
        profiles_path=Path(
            Path.home() / "Library/Application Support/QGIS/QGIS3/profiles/"
        )
    ),
    "linux": OSConfiguration(
        profiles_path=Path(Path.home() / ".local/share/QGIS/QGIS3/profiles/")
    ),
    "windows": OSConfiguration(
        profiles_path=Path(expandvars("%APPDATA%/Roaming/QGIS/QGIS3/profiles"))
    ),
}



#! python3  # noqa: E265

"""
    Sub-command in charge of cleaning up temporary files.

"""

# #############################################################################
# ########## Libraries #############
# ##################################

# Standard library
import logging
from datetime import datetime, timedelta
from pathlib import Path
from timeit import default_timer

# 3rd party library
import click
from send2trash import send2trash

# submodules
from qgis_deployment_toolbelt.utils.bouncer import exit_cli_success

# #############################################################################
# ########## Globals ###############
# ##################################

# chronometer
START_TIME = default_timer()

# logs
logger = logging.getLogger(__name__)

# default CLI context.
# See: https://click.palletsprojects.com/en/7.x/commands/#context-defaults
CONTEXT_SETTINGS = dict(obj={})


# #############################################################################
# ####### Command-line ############
# #################################
@click.command(name="clean")
@click.option(
    "-rm",
    "--remove-empty-folders",
    help="Remove empty folders.",
    is_flag=True,
    default=True,
    show_default=True,
)
@click.pass_context
def clean(cli_context: click.Context, remove_empty_folders: bool):
    """Delete logs and output folders older than the frequency set in the \
    configuration: `CLEAN_FREQUENCY`.

    \f
    :param click.core.Context cli_context: Click context
    :param bool remove_empty_folders: if passed, empty folders will be deleted too
    """
    logger.info("CLEAN started after {:5.2f}s.".format(default_timer() - START_TIME))

    # extract values from CLI context
    logs_folder = cli_context.obj.get("FOLDER_LOGS")
    config_dict = cli_context.obj.get("SETTINGS_DICT")

    logger.info(
        "Suppression des fichiers antérieurs à {} jours.".format(
            config_dict.get("global").get("clean_frequency_days")
        )
    )
    date_ref_days_ago = datetime.now() - timedelta(
        days=int(config_dict.get("global").get("clean_frequency_days"))
    )

    # logs folder
    li_folders_logs = Path(logs_folder).iterdir()

    # go deleting
    with click.progressbar(
        list(li_folders_logs),
        label="Removing log folders",
    ) as log_folders:
        for subobj in log_folders:
            # compare last modification date
            if datetime.fromtimestamp(subobj.stat().st_mtime) < date_ref_days_ago:
                logger.debug(
                    "CLEANER - LOGS - Detected folder/file outdated: {}".format(subobj)
                )
                try:
                    send2trash(str(subobj.resolve()))
                except Exception as e:
                    logger.warning(
                        "Unable to delete: {}. Original error: {}".format(
                            subobj.resolve(), e
                        )
                    )
            # handle empty folders
            if (
                remove_empty_folders
                and subobj.is_dir()
                and not list(subobj.glob("**/*"))
            ):
                logger.info(
                    "Empty folder spotted: {}. Sending it to the trash...".format(
                        str(subobj.resolve())
                    )
                )
                try:
                    send2trash(str(subobj.resolve()))
                except Exception as e:
                    logger.warning(
                        "Unable to delete: {}. Original error: {}".format(
                            subobj.resolve(), e
                        )
                    )

    # ending
    exit_cli_success(
        message="CLEAN completed after {:5.2f}s.".format(default_timer() - START_TIME),
        abort=False,
    )


# #############################################################################
# ##### Stand alone program ########
# ##################################
if __name__ == "__main__":
    """Standalone execution."""
    # launch cli
    clean(obj={})



#! python3  # noqa: E265

"""
    Read and validate scenario files.

    Author: Julien Moura (https://github.com/guts, Oslandia)
"""


# #############################################################################
# ########## Libraries #############
# ##################################

# Standard library
import importlib
import logging
from pathlib import Path
from typing import List, Tuple

from qgis_deployment_toolbelt.jobs.job_environment_variables import (
    JobEnvironmentVariables,
)
from qgis_deployment_toolbelt.jobs.job_profiles_synchronizer import (
    JobProfilesDownloader,
)

# #############################################################################
# ########## Globals ###############
# ##################################

# logs
logger = logging.getLogger(__name__)

# #############################################################################
# ########## Classes ###############
# ##################################


class JobsOrchestrator:
    """Orchestrate jobs."""

    JOBS = (JobEnvironmentVariables, JobProfilesDownloader)
    PACKAGE_NAME: str = "qgis_deployment_toolbelt.jobs"

    def __init__(self) -> None:
        pass

    @property
    def available_jobs(self) -> List[Path]:
        """List all available jobs."""
        # job modules
        return list(Path(__file__).parent.glob("job_*.py"))

    @property
    def jobs_ids(self) -> Tuple[str]:
        """Returns available jobs ID.

        :return Tuple[str]: jobs ids
        """
        return tuple([job.ID for job in self.JOBS])

    def get_job_module_from_id(self, job_id: str) -> object:
        """Get job class from id."""
        for job in self.JOBS:
            if job.ID == job_id:
                return job

    def init_job_class_from_id(self, job_id: str, options: dict) -> object:
        """Get job class from id and instanciate it with options."""
        return self.get_job_module_from_id(job_id)(options)

    def import_job(self, job_name: str):
        """Import a job."""
        return importlib.import_module(name=job_name, package=self.PACKAGE_NAME)

    def import_all_jobs(self) -> None:
        """Import all jobs."""
        for module in self.available_jobs:
            self.JOBS[module.stem] = self.import_job(module.stem)


# #############################################################################
# ##### Stand alone program ########
# ##################################

if __name__ == "__main__":
    """Standalone execution."""
    import inspect
    from pprint import pprint

    from generic import JobBase

    # mod = importlib.import_module(
    #     name="job_environment_variables", package="qgis_deployment_toolbelt.jobs"
    # )
    # print(mod.__doc__, mod.__name__, dir(mod))

    orchestrator = JobsOrchestrator()
    jobs_available = orchestrator.available_jobs

    j = orchestrator.import_job("job_environment_variables")
    print(j.__doc__, j.__name__, dir(j))

    # orchestrator.import_all_jobs()

    # pprint(orchestrator.JOBS)

    # for k, v in orchestrator.JOBS.items():
    #     print(v, type(v), dir(v))
    #     for name_local in dir(v):
    #         if name_local.startswith("__"):
    #             continue
    #         module_class = getattr(v, name_local)
    #         print(str(module_class), type(module_class))
    # #         print(name_local, isclass(getattr(v, name_local)))
    #         # print(name_local, issubclass(getattr(v, name_local), JobBase))
    #         print(module_class, issubclass(getattr(v, module_class), JobBase))
    # #         print(name_local, isinstance(getattr(v, name_local), JobBase))
    #         if isclass(getattr(v, name_local)) and isinstance(getattr(v, name_local), JobBase):
    #             print("test " + name_local)
    #             # print(f"{k} - {name_local.ID}")
    #         # for name_local in dir(v):
    #         #     if inspect.isclass(getattr(v, name_local)):
    #         #         print(f'{name_local} is a class')
    #         #         MysteriousClass = getattr(v, name_local)
    #         #         mysterious_object = MysteriousClass()

    # READ https://julienharbulot.com/python-dynamical-import.html

    # from inspect import isclass
    # from pkgutil import iter_modules
    # from pathlib import Path
    # from importlib import import_module

    # # iterate through the modules in the current package
    # package_dir = Path(__file__).resolve().parent
    # for (_, module_name, _) in iter_modules([package_dir]):

    #     # import the module and iterate through its attributes
    #     module = import_module(f"{__name__}.{module_name}")
    #     for attribute_name in dir(module):
    #         attribute = getattr(module, attribute_name)

    #         if issubclass(attribute, JobBase):
    #             # Add the class to this package's variables
    #             globals()[attribute_name] = attribute
